
#  Copyright (c) 2013-2014, The University of Chicago
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of The University of Chicago nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.

from github import Github, InputGitAuthor
from github.GithubException import GithubException

import git
import os.path
import pytz
from datetime import datetime

import chisubmit.core
from chisubmit.core import ChisubmitException
from git.exc import GitCommandError

class GithubConnection(object):
    def __init__(self, access_token, organization):
        self.gh = Github(access_token)

        try:
            self.organization = self.gh.get_organization(organization)
        except GithubException as ge:
            if ge.status == 401:
                raise ChisubmitException("Invalid Github Credentials", ge)
            elif ge.status == 404:
                raise ChisubmitException("Organization %s does not exist" % organization, ge)            
            else:
                raise ChisubmitException("Unexpected error accessing organization %s (%i: %s)" % (organization, ge.status, ge.data["message"]), ge)            

    def create_team_repository(self, course, team, fail_if_exists=True, private=True):
        repo_name = "%s-%s" % (course.id, team.id)
        team_name = repo_name
        student_names = ", ".join(["%s %s" % (s.first_name, s.last_name) for s in team.students])
        repo_description = "%s: Team %s (%s)" % (course.name, team.id, student_names)
        github_instructors = self.__get_team_by_name(course.github_admins_team)
        github_students = []
        
        # Make sure users exist
        for s in team.students:
            github_student = self.__get_user(s.github_id)
            
            if github_student is None:
                raise ChisubmitException("GitHub user '%s' does not exist " % (s.github_id))
            
            github_students.append(github_student)
        
        github_repo = self.__get_repository(repo_name)
        
        if github_repo is None:
            try:
                github_repo = self.organization.create_repo(repo_name, description=repo_description, private=private)
            except GithubException as ge:
                raise ChisubmitException("Unexpected exception creating repository %s (%i: %s)" % (repo_name, ge.status, ge.data["message"]), ge)
        else:
            if fail_if_exists:
                raise ChisubmitException("Repository %s already exists" % repo_name)
        
        try:
            github_instructors.add_to_repos(github_repo)
        except GithubException as ge:
            raise ChisubmitException("Unexpected exception adding repository to Instructors team (%i: %s)" % (ge.status, ge.data["message"]), ge)
        
        github_team = self.create_team(team_name, [github_repo], "push", fail_if_exists)
        
        for github_student in github_students:
            try:
                github_team.add_to_members(github_student)
            except GithubException as ge:
                raise ChisubmitException("Unexpected exception adding user %s to team (%i: %s)" % (s.github_id, ge.status, ge.data["message"]), ge)
            
        # Update team information only once everything has completed
        # successfully
        team.github_repo = repo_name
        team.github_team = team_name
            

    def update_team_repository(self, team):
        github_team = self.__get_team_by_name(team.github_team)

        for s in team.students:
            github_student = self.__get_user(s.github_id)
            try:
                github_team.add_to_members(github_student)
            except GithubException as ge:
                raise ChisubmitException("Unexpected exception adding user %s to team (%i: %s)" % (s.github_id, ge.status, ge.data["message"]))


    def create_team(self, team_name, repos, permissions, fail_if_exists = True):
        github_team = self.__get_team_by_name(team_name)
        
        if github_team is not None:
            if fail_if_exists:
                raise ChisubmitException("Team %s already exists." % team_name)
            else:
                return github_team
        else:
            try:
                github_team = self.organization.create_team(team_name, repos, permissions)
                return github_team
            except GithubException as ge:
                raise ChisubmitException("Unexpected exception creating team %s (%i: %s)" % (team_name, ge.status, ge.data["message"]), ge)


    def create_submission_tag(self, team, tag_name, tag_message, commit_sha):
        github_repo = self.organization.get_repo(team.github_repo)
        
        commit = self.get_commit(team, commit_sha)
        
        this_user = self.gh.get_user()

        if this_user.name is None:
            user_name = "Team %s" % team.id
        else:
            user_name = this_user.name

        if this_user.email is None:
            user_email = "unspecified@example.org"
        else:
            user_email = this_user.email

        tz = pytz.timezone("America/Chicago")
        dt = tz.localize(datetime.now())
        iu = InputGitAuthor(user_name, user_email, dt.isoformat())
        
        tag = github_repo.create_git_tag(tag_name, tag_message, commit.sha, "commit", iu)
        ref = github_repo.create_git_ref("refs/tags/" + tag.tag, tag.sha)
        
    def update_submission_tag(self, team, tag_name, tag_message, commit_sha):
        submission_tag_ref = self.get_submission_tag_ref(team, tag_name)
        submission_tag_ref.delete()
        
        self.create_submission_tag(team, tag_name, tag_message, commit_sha)
        
    def get_submission_tag_ref(self, team, tag_name):
        github_repo = self.organization.get_repo(team.github_repo)
        
        try:
            submission_tag_ref = github_repo.get_git_ref("tags/" + tag_name)
        except GithubException as ge:
            if ge.status == 404:
                return None
            else:
                raise ChisubmitException("Unexpected error when fetching tag %s (%i: %s)" % (tag_name, ge.status, ge.data["message"]), ge)            
        
        return submission_tag_ref

            
    def get_submission_tag(self, team, tag_name):
        github_repo = self.organization.get_repo(team.github_repo)
        
        submission_tag_ref = self.get_submission_tag_ref(team, tag_name)
        
        if submission_tag_ref is None:
            return None
                
        submission_tag = github_repo.get_git_tag(submission_tag_ref.object.sha)
        
        return submission_tag  

    def delete_team_repository(self, team):
        try:
            github_repo = self.organization.get_repo(team.github_repo)
        except GithubException as ge:
            raise ChisubmitException("Unexpected exception fetching repository %s (%i: %s)" % (team.github_repo, ge.status, ge.data["message"]), ge)
        
        try:
            github_repo.delete()
        except GithubException as ge:
            raise ChisubmitException("Unexpected exception deleting repository %s (%i: %s)" % (team.github_repo, ge.status, ge.data["message"]), ge)
        
        team.github_repo = None
        
        github_team = self.__get_team_by_name(team.github_team)

        try:
            github_team.delete()
        except GithubException as ge:
            raise ChisubmitException("Unexpected exception deleting team %s (%i: %s)" % (team.github_team, ge.status, ge.data["message"]), ge)

        team.github_team = None

    def get_commit(self, team, commit_sha):
        try:
            github_repo = self.organization.get_repo(team.github_repo)
            commit = github_repo.get_commit(commit_sha)
            return commit
        except GithubException as ge:
            if ge.status == 404:
                return None
            else:
                raise ChisubmitException("Unexpected error when fetching commit %s (%i: %s)" % (commit_sha, ge.status, ge.data["message"]), ge)            


    def repository_exists(self, github_repo):
        r = self.__get_repository(github_repo)
        
        return (r is not None)
        

    def __get_user(self, username):
        try:
            user = self.gh.get_user(username)
            return user
        except GithubException as ge:
            if ge.status == 404:
                return None
            else:
                raise ChisubmitException("Unexpected error with user %s (%i: %s)" % (username, ge.status, ge.data["message"]), ge)            


    def __get_repository(self, repository_name):
        try:
            repository = self.organization.get_repo(repository_name)
            return repository
        except GithubException as ge:
            if ge.status == 404:
                return None
            else:
                raise ChisubmitException("Unexpected error with repository %s (%i: %s)" % (repository_name, ge.status, ge.data["message"]), ge)            


    def __get_team_by_name(self, team_name):
        try:
            teams = self.organization.get_teams()
            for t in teams:
                if t.name == team_name:
                    return t
            return None
        except GithubException as ge:
            raise ChisubmitException("Unexpected error with team %s (%i: %s)" % (team_name, ge.status, ge.data["message"]), ge)
        
    @staticmethod
    def get_token(username, password, delete = False):
        gh = Github(username, password)
        token = None
        
        try:
            u = gh.get_user()
            
            scopes = ['user', 'public_repo', 'repo', 'gist']
            note = "Created by chisubmit."
            
            if delete:
                scopes.append("delete_repo")
                note += " Has delete permissions."
                
            auth = u.create_authorization(scopes = scopes, note = note)
            token = auth.token
        except GithubException as ge:
            if ge.status == 401:
                return None
            else:
                raise ChisubmitException("Unexpected error creating authorization token (%i: %s)" % (ge.status, ge.data["message"]), ge)            
        
        return token
    
class LocalGitRepo(object):
    def __init__(self, directory):
        self.repo = git.Repo(directory)
        
        self.remotes = dict([(r.name, r) for r in self.repo.remotes])

    @classmethod
    def create_repo(cls, directory, clone_from_url = None, remotes = []):
        if clone_from_url is None:
            # TODO: Create empty repo
            pass
        else:
            repo = git.Repo.clone_from(clone_from_url, directory)
            
            for remote_name, remote_url in remotes:
                repo.create_remote(remote_name, remote_url)
            
            return cls(directory)

    def fetch(self, remote_name):
        self.remotes[remote_name].fetch()

    def reset_branch(self, remote_name, branch):
        branch_refpath = "refs/heads/%s" % branch
        remote_branch_refpath = "refs/remotes/%s/%s" % (remote_name, branch)
        
        branch_head = self.__get_head(branch_refpath)
        remote_branch = self.__get_ref(remote_branch_refpath)
        
        if branch_head is None:
            raise ChisubmitException("No such branch: %s" % branch)

        if remote_branch is None:
            raise ChisubmitException("No such remote branch: %s" % branch)

        if self.repo.head.ref != branch_head:
            try:
                branch_head.checkout()
            except GitCommandError, gce:
                print gce
                raise ChisubmitException("Error checking out")
        
        self.repo.head.reset(remote_branch.commit, index=True, working_tree=True)

    def has_branch(self, branch):
        return (self.__get_branch(branch) is not None)
    
    def checkout_branch(self, branch):
        branch_refpath = "refs/heads/%s" % branch
        branch_head = self.__get_head(branch_refpath)
            
        if branch_head is None:
            raise ChisubmitException("No such branch: %s" % branch)    
        
        try:
            branch_head.checkout()
        except GitCommandError, gce:
            raise ChisubmitException("Error checking out")                

    def get_tag(self, tag):
        tags = [t for t in self.repo.tags if t.name == tag]
        
        if len(tags) == 0:
            return None
        else:
            return tags[0]           

    def has_tag(self, tag):
        return (self.get_tag(tag) is not None)
    
    def create_branch(self, branch, commit):
        self.repo.create_head(branch, commit)

    def push(self, remote_name, branch):
        self.remotes[remote_name].push("%s:%s" % (branch, branch))

    def pull(self, remote_name, branch):
        self.remotes[remote_name].pull("%s:%s" % (branch, branch))        


    def __get_head(self, path):
        heads = [h for h in self.repo.heads if h.path == path]
        
        if len(heads) == 0:
            return None
        else:
            return heads[0]

    def __get_branch(self, branch):
        branches = [b for b in self.repo.branches if b.name == branch]
        
        if len(branches) == 0:
            return None
        else:
            return branches[0]   
            
    def __get_ref(self, path):
        refs = [r for r in self.repo.refs if r.path == path]
        
        if len(refs) == 0:
            return None
        else:
            return refs[0]            
        
class GradingGitRepo(object):
    def __init__(self, team, repo):
        self.team = team        
        self.repo = repo
    
    @classmethod
    def get_grading_repo(cls, course, team):
        repo_path = cls.get_grading_repo_path(course, team)
        if not os.path.exists(repo_path):
            return None 
        else:
            repo = LocalGitRepo(repo_path)
            return cls(team, repo)
    
    @classmethod
    def create_grading_repo(cls, course, team):
        repo_path = cls.get_grading_repo_path(course, team)
        gh_url = cls.get_team_gh_repo_url(course, team)
        staging_url = cls.get_team_staging_repo_url(course, team)
        repo = LocalGitRepo.create_repo(repo_path, clone_from_url = gh_url, remotes = [("staging", staging_url)])
        return cls(team, repo)        
    
    def sync(self):
        self.repo.fetch("origin")
        self.repo.reset_branch("origin", "master")
        
    def create_grading_branch(self, project):
        tag = self.repo.get_tag(project.id)
        if tag is None:
            raise ChisubmitException("%s repository does not have a %s tag" % (self.team.id, project.id))
        
        branch_name = project.get_grading_branch_name()
        if self.repo.has_branch(branch_name):
            raise ChisubmitException("%s repository already has a %s branch" % (self.team.id, branch_name))
        
        self.repo.create_branch(branch_name, tag.commit)
        self.repo.checkout_branch(branch_name)
        
        
    def push_grading_branch_to_github(self, project):
        self.__push_grading_branch(project, "origin")

    def push_grading_branch_to_staging(self, project):
        self.__push_grading_branch(project, "staging", push_master = True)

    def pull_grading_branch_from_github(self, project):
        self.__pull_grading_branch(project, "origin")

    def pull_grading_branch_from_staging(self, project):
        self.__pull_grading_branch(project, "staging")

    def __push_grading_branch(self, project, remote_name, push_master = False):
        branch_name = project.get_grading_branch_name()
        
        if not self.repo.has_branch(branch_name):
            raise ChisubmitException("%s repository does not have a %s branch" % (self.team.id, branch_name))
        
        if push_master:
            self.repo.push(remote_name, "master")
            
        self.repo.push(remote_name, branch_name)
        
    def __pull_grading_branch(self, project, remote_name):
        branch_name = project.get_grading_branch_name()
        self.repo.pull(remote_name, branch_name)

    @staticmethod
    def get_grading_repo_path(course, team):
        return "%s/repositories/%s/%s" % (chisubmit.core.chisubmit_dir, course.id, team.id)
    
    @staticmethod
    def get_team_gh_repo_url(course, team):        
        return "git@github.com:%s/%s.git" % (course.github_organization, team.github_repo)       

    @staticmethod
    def get_team_staging_repo_url(course, team):
        if course.git_staging_hostname is None or course.git_staging_username is None:
            return None
        else:
            return "%s@%s:%s.git" % (course.git_staging_username, course.git_staging_hostname, team.github_repo)
                



#  Copyright (c) 2013-2014, The University of Chicago
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of The University of Chicago nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.

from chisubmit.common.utils import create_subparser
from chisubmit.common import CHISUBMIT_SUCCESS, CHISUBMIT_FAIL
from chisubmit.core import ChisubmitException, handle_unexpected_exception

def create_shell_subparsers(subparsers):
    create_subparser(subparsers, "shell", cli_do__shell)
    
    
def cli_do__shell(course, args):
    try:
        from IPython import embed, __version__
        from IPython.config.loader import Config
        if __version__ < "1.1.0":
            print "You need IPython (>= 1.1.0) to run the chisubmit shell"
            return CHISUBMIT_FAIL
    except ImportError, ie:
        print "You need IPython (>= 1.1.0) to run the chisubmit shell"
        
    try:
        cfg = Config()
        cfg.TerminalInteractiveShell.banner1 = """
                      WELCOME TO THE CHISUBMIT SHELL
    
    Course: %s
    
    This is an IPython shell with the chisubmit data structures preloaded. 
    You can access the chisubmit objects through variable 'course'.
    
    Note: Any changes you make will NOT be saved. Use course.save() to save
    any changes you make to the chisubmit objects.
    
    """ % (course.name)
    
        prompt_config = cfg.PromptManager
        prompt_config.in_template = 'chisubmit> '
        prompt_config.in2_template = '   .\\D.> '
        prompt_config.out_template = '         > '
        embed(config = cfg)
    except ChisubmitException, ce:
        raise ce # Propagate upwards, it will be handled by chisubmit_cmd
    except Exception, e:
        handle_unexpected_exception()
        
    return CHISUBMIT_SUCCESS



#  Copyright (c) 2013-2014, The University of Chicago
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of The University of Chicago nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.

import chisubmit.core

from chisubmit.common.utils import create_subparser
from chisubmit.core import handle_unexpected_exception
from chisubmit.core.model import Course
from chisubmit.core.repos import GithubConnection
from chisubmit.common import CHISUBMIT_SUCCESS
from chisubmit.core import ChisubmitException

def create_course_subparsers(subparsers):
    subparser = create_subparser(subparsers, "course-create", cli_do__course_create)
    subparser.add_argument('--make-default', action="store_true", dest="make_default")
    subparser.add_argument('id', type=str)
    subparser.add_argument('name', type=str)
    subparser.add_argument('extensions', type=int)

    subparser = create_subparser(subparsers, "course-install", cli_do__course_install)
    subparser.add_argument('--make-default', action="store_true", dest="make_default")
    subparser.add_argument('filename', type=str)
    
    subparser = create_subparser(subparsers, "course-github-settings", cli_do__course_github_settings)
    subparser.add_argument('organization', type=str)

    subparser = create_subparser(subparsers, "course-git-staging-settings", cli_do__course_git_staging_settings)
    subparser.add_argument('git_username', type=str)
    subparser.add_argument('git_hostname', type=str)

    subparser = create_subparser(subparsers, "course-generate-distributable", cli_do__course_generate_distributable)
    subparser.add_argument('filename', type=str)
    
    
def cli_do__course_create(course, args):
    course = Course(course_id = args.id,
                    name = args.name,
                    extensions = args.extensions)
    
    try:
        course.course_file = chisubmit.core.get_course_filename(course.id)
        course.save()
        
        if args.make_default:
            chisubmit.core.set_default_course(args.id)
    except ChisubmitException, ce:
        raise ce # Propagate upwards, it will be handled by chisubmit_cmd
    except Exception, e:
        handle_unexpected_exception()
        
    return CHISUBMIT_SUCCESS


def cli_do__course_install(course, args):
    if args.filename.startswith("http"):
        new_course = Course.from_url(args.filename)
    else:
        f = open(args.filename)
        new_course = Course.from_file(f)
        f.close()
        
    try:
        new_course.course_file = chisubmit.core.get_course_filename(new_course.id)
        new_course.save()
        
        if args.make_default:
            chisubmit.core.set_default_course(new_course.id)
    except ChisubmitException, ce:
        raise ce # Propagate upwards, it will be handled by chisubmit_cmd
    except Exception, e:
        handle_unexpected_exception()
        
    return CHISUBMIT_SUCCESS


def cli_do__course_github_settings(course, args):
    course.github_organization = args.organization
    
    try:
        # Try connecting to GitHub
        github_access_token = chisubmit.core.get_github_token()
        gh = GithubConnection(github_access_token, course.github_organization)
        
        # Create GitHub team to contain admins for this course
        course.github_admins_team = "%s-admins" % course.id
        gh.create_team(course.github_admins_team, [], "admin", fail_if_exists = False)    
    except ChisubmitException, ce:
        raise ce # Propagate upwards, it will be handled by chisubmit_cmd
    except Exception, e:
        handle_unexpected_exception()
    
    return CHISUBMIT_SUCCESS


def cli_do__course_git_staging_settings(course, args):
    course.git_staging_username = args.git_username
    course.git_staging_hostname = args.git_hostname
    
    return CHISUBMIT_SUCCESS


def cli_do__course_generate_distributable(course, args):
    course.github_admins_team = None
    course.git_staging_username = None
    course.git_staging_hostname = None
    course.students = {}
    course.teams = {}
    
    try:
        course.save(args.filename)
    except ChisubmitException, ce:
        raise ce # Propagate upwards, it will be handled by chisubmit_cmd
    except Exception, e:
        handle_unexpected_exception()
        
    return CHISUBMIT_SUCCESS
    



#  Copyright (c) 2013-2014, The University of Chicago
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of The University of Chicago nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.

import sys
import traceback
import datetime
import os.path

from argparse import ArgumentParser

import chisubmit.core
import chisubmit.common.log as log
from chisubmit import RELEASE
from chisubmit.core.model import Course, Project, Student,\
    ChisubmitModelException
from chisubmit.cli.course import *
from chisubmit.cli.student import *
from chisubmit.cli.team import create_team_subparsers
from chisubmit.cli.project import create_project_subparsers
from chisubmit.cli.submit import create_submit_subparsers
from chisubmit.core import ChisubmitException
from chisubmit.common import CHISUBMIT_FAIL
from chisubmit.cli.shell import create_shell_subparsers
from chisubmit.cli.grader import create_grader_subparsers
from chisubmit.cli.gh import create_gh_subparsers

SUBCOMMANDS_NO_COURSE = ['course-create', 'course-install', 'gh-token-create']
SUBCOMMANDS_DONT_SAVE = ['course-create', 'course-install', 'course-generate-distributable', 'gh-token-create', 'shell']

            
def chisubmit_cmd(argv=None):
    if argv is None:
        argv = sys.argv[1:]
        
    # Setup argument parser
    parser = ArgumentParser(description="chisubmit")
    parser.add_argument('--config', type=str, default=None)
    parser.add_argument('--dir', type=str, default=None)
    parser.add_argument('--noop', action="store_true")
    parser.add_argument('--course', type=str, default=None)
    parser.add_argument('--verbose', action="store_true")
    parser.add_argument('--version', action='version', version="chisubmit %s" % RELEASE)
    parser.add_argument('--debug', action="store_true")

    subparsers = parser.add_subparsers(dest="subcommand")
    
    create_course_subparsers(subparsers)
    create_project_subparsers(subparsers)
    create_student_subparsers(subparsers)
    create_team_subparsers(subparsers)
    create_submit_subparsers(subparsers)
    create_grader_subparsers(subparsers)
    create_shell_subparsers(subparsers)
    create_gh_subparsers(subparsers)
    
    args = parser.parse_args(args = argv)
    
    chisubmit.core.init_chisubmit(args.dir, args.config)
    log.init_logging(args.verbose, args.debug)

    if args.subcommand not in SUBCOMMANDS_NO_COURSE:
        if args.course is not None:
            course_id = args.course
        else:
            course_id = chisubmit.core.get_default_course()
        
        if course_id is None:
            print "ERROR: No course specified with --course and no default course in configuration file"
            exit(CHISUBMIT_FAIL)
        else:
            course_obj = Course.from_course_id(course_id)
            if course_obj is None:
                print "ERROR: Course '%s' does not exist" % course_id
                exit(CHISUBMIT_FAIL)
    else:
        course_id, course_obj = None, None

    if not args.noop:
        try:
            rc = args.func(course_obj, args)
        except ChisubmitException, ce:
            print "ERROR: %s" % ce.message
            if args.debug:
                ce.print_exception()
            exit(CHISUBMIT_FAIL)
        except ChisubmitModelException, cme:
            print "ERROR: %s" % cme.message
            exit(CHISUBMIT_FAIL)
        except Exception, e:
            handle_unexpected_exception()

    if args.subcommand not in SUBCOMMANDS_DONT_SAVE:
        course_obj.save()

    return rc




#  Copyright (c) 2013-2014, The University of Chicago
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of The University of Chicago nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.

import chisubmit.core

from chisubmit.common.utils import create_subparser
from chisubmit.core.model import Team
from chisubmit.core.repos import GithubConnection
from chisubmit.common import CHISUBMIT_SUCCESS, CHISUBMIT_FAIL
from chisubmit.core import ChisubmitException, handle_unexpected_exception
import os.path
import smtplib
from string import Template

def create_team_subparsers(subparsers):
    subparser = create_subparser(subparsers, "team-create", cli_do__team_create)
    subparser.add_argument('team_id', type=str)

    subparser = create_subparser(subparsers, "team-list", cli_do__team_list)
    subparser.add_argument('--ids', action="store_true")
    
    subparser = create_subparser(subparsers, "team-student-add", cli_do__team_student_add)
    subparser.add_argument('team_id', type=str)
    subparser.add_argument('student_id', type=str)

    subparser = create_subparser(subparsers, "team-project-add", cli_do__team_project_add)
    subparser.add_argument('team_id', type=str)
    subparser.add_argument('project_id', type=str)
    
    subparser = create_subparser(subparsers, "team-gh-repo-create", cli_do__team_gh_repo_create)
    subparser.add_argument('team_id', type=str)
    subparser.add_argument('--ignore-existing', action="store_true", dest="ignore_existing")
    
    subparser = create_subparser(subparsers, "team-gh-repo-update", cli_do__team_gh_repo_update)
    subparser.add_argument('team_id', type=str)    

    subparser = create_subparser(subparsers, "team-gh-repo-remove", cli_do__team_gh_repo_remove)
    subparser.add_argument('team_id', type=str)

    subparser = create_subparser(subparsers, "team-gh-repo-check", cli_do__team_gh_repo_check)
    subparser.add_argument('team_id', type=str)

    subparser = create_subparser(subparsers, "team-gh-repo-set", cli_do__team_gh_repo_set)
    subparser.add_argument('team_id', type=str)
    subparser.add_argument('github_repo', type=str)

    subparser = create_subparser(subparsers, "team-gh-repo-email", cli_do__team_gh_repo_email)
    subparser.add_argument('team_id', type=str)
    subparser.add_argument('email_from', type=str)
    subparser.add_argument('template', type=str)
    subparser.add_argument('--force', action="store_true")
    subparser.add_argument('--dry-run', action="store_true")


def cli_do__team_create(course, args):
    team = Team(team_id = args.team_id)
    course.add_team(team)
        
    return CHISUBMIT_SUCCESS

def cli_do__team_list(course, args):
    team_ids = course.teams.keys()
    team_ids.sort()
    
    for team_id in team_ids:
        if args.ids:
            print team_id
        else:
            team = course.teams[team_id]
            
            fields = [team.id, `team.active`, team.github_repo, team.github_team, `team.github_email_sent`]
                        
            print "\t".join(fields)

    return CHISUBMIT_SUCCESS

    
def cli_do__team_student_add(course, args):
    student = course.get_student(args.student_id)
    if student is None:
        print "Student %s does not exist" % args.student_id
        return CHISUBMIT_FAIL
    
    team = course.get_team(args.team_id)
    if team is None:
        print "Team %s does not exist" % args.team_id
        return CHISUBMIT_FAIL       
    
    team.add_student(student)   

    return CHISUBMIT_SUCCESS

    
def cli_do__team_project_add(course, args):
    project = course.get_project(args.project_id)
    if project is None:
        print "Project %s does not exist" % args.project_id
        return CHISUBMIT_FAIL    
    
    team = course.get_team(args.team_id)
    if team is None:
        print "Team %s does not exist"
        return CHISUBMIT_FAIL
    
    team.add_project(project)                

    return CHISUBMIT_SUCCESS

    
def cli_do__team_gh_repo_create(course, args):
    team = course.get_team(args.team_id)
    if team is None:
        print "Team %s does not exist" % args.team_id
        return CHISUBMIT_FAIL

    if team.github_repo is not None and not args.ignore_existing:
        print "Repository for team %s has already been created." % team.id
        print "Maybe you meant to run team-repo-update?"
        return CHISUBMIT_FAIL

    github_access_token = chisubmit.core.get_github_token()

    if github_access_token is None:
        print "You have not created a GitHub access token."
        print "You can create one using 'chisubmit gh-token-create'"
        return CHISUBMIT_FAIL    
    
    try:
        gh = GithubConnection(github_access_token, course.github_organization)
            
        gh.create_team_repository(course, team, fail_if_exists = not args.ignore_existing)
    except ChisubmitException, ce:
        raise ce # Propagate upwards, it will be handled by chisubmit_cmd
    except Exception, e:
        handle_unexpected_exception()
        
    return CHISUBMIT_SUCCESS

    
def cli_do__team_gh_repo_update(course, args):
    team = course.get_team(args.team_id)
    if team is None:
        print "Team %s does not exist" % args.team_id
        return CHISUBMIT_FAIL
    
    if team.github_repo is None:
        print "Team %s does not have a repository." % team.id
        return CHISUBMIT_FAIL

    github_access_token = chisubmit.core.get_github_token()

    if github_access_token is None:
        print "You have not created a GitHub access token."
        print "You can create one using 'chisubmit gh-token-create'"
        return CHISUBMIT_FAIL    
    
    try:    
        gh = GithubConnection(github_access_token, course.github_organization)
        gh.update_team_repository(team)    
    except ChisubmitException, ce:
        raise ce # Propagate upwards, it will be handled by chisubmit_cmd
    except Exception, e:
        handle_unexpected_exception()

    return CHISUBMIT_SUCCESS

    
def cli_do__team_gh_repo_remove(course, args):
    team = course.get_team(args.team_id)
    if team is None:
        print "Team %s does not exist" % args.team_id
        return CHISUBMIT_FAIL
    
    if team.github_repo is None:
        print "Team %s does not have a repository." % team.id
        return CHISUBMIT_FAIL

    github_access_token = chisubmit.core.get_github_delete_token()
    
    if github_access_token is None:
        print "No GitHub access token with delete permissions found."
        print "You can create one with 'chisubmit gh-token-create --delete"
        return CHISUBMIT_FAIL
        
    try:
        gh = GithubConnection(github_access_token, course.github_organization)
        gh.delete_team_repository(team)
    except ChisubmitException, ce:
        raise ce # Propagate upwards, it will be handled by chisubmit_cmd
    except Exception, e:
        handle_unexpected_exception()

    return CHISUBMIT_SUCCESS

def cli_do__team_gh_repo_check(course, args):
    team = course.get_team(args.team_id)
    if team is None:
        print "Team %s does not exist" % args.team_id
        return CHISUBMIT_FAIL
    
    if team.github_repo is None:
        print "Team %s does not have a repository." % team.id
        return CHISUBMIT_FAIL

    github_access_token = chisubmit.core.get_github_token()

    if github_access_token is None:
        print "You have not created a GitHub access token."
        print "You can create one using 'chisubmit gh-token-create'"
        return CHISUBMIT_FAIL    
        
    try:
        gh = GithubConnection(github_access_token, course.github_organization)
    except ChisubmitException, ce:
        print "Unable to connect to GitHub: %s" % ce.message
        return CHISUBMIT_FAIL
        
    if not gh.repository_exists(team.github_repo):
        print "The repository '%s' does not exist or you do not have permission to access it." % team.github_repo
        return CHISUBMIT_FAIL

    # TODO: Check that the user actually has push access
    print "Repository '%s' exists and you have access to it." % team.github_repo
    print "GitHub URL: https://github.com/%s/%s" % (course.github_organization, team.github_repo)

    return CHISUBMIT_SUCCESS

def cli_do__team_gh_repo_set(course, args):
    team = course.get_team(args.team_id)
    if team is None:
        print "Team %s does not exist" % args.team_id
        return CHISUBMIT_FAIL
    
    team.github_repo = args.github_repo
    
    return CHISUBMIT_SUCCESS


def cli_do__team_gh_repo_email(course, args):
    team = course.get_team(args.team_id)
    if team is None:
        print "Team %s does not exist" % args.team_id
        return CHISUBMIT_FAIL
    
    if team.github_email_sent and not args.force:
        print "E-mail to team %s has already been sent. Use --force to send anyways." % team.id
        return CHISUBMIT_FAIL
    
    if team.github_repo is None:
        print "Team %s does not have a repository." % team.id
        return CHISUBMIT_FAIL

    if not os.path.exists(args.template):
        print "File %s does not exists"
        return CHISUBMIT_FAIL
       
    smtp_server = smtplib.SMTP(chisubmit.core.chisubmit_conf.get("smtp", "server"))
    email_template = Template(open(args.template).read()) 
    email_to = ["%s %s <%s>" % (s.first_name, s.last_name, s.email) for s in team.students]
    email_rcpt = [s.email for s in team.students] + [args.email_from]
    github_repo = "https://github.com/%s/%s" % (course.github_organization, team.github_repo)
    email_message = email_template.substitute(sender = args.email_from, 
                                              recipients = ", ".join(email_to), 
                                              subject = "New GitHub repository: %s" % team.github_repo, 
                                              github_repo = github_repo)
    
    if not args.dry_run:
        smtp_server.sendmail(args.email_from, email_rcpt, email_message)
        team.github_email_sent = True
        print "E-mail sent to team %s" % team.id
    else:
        print "Dry run requested. This is the message that would have been sent:"
        print
        print email_message
        




#  Copyright (c) 2013-2014, The University of Chicago
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of The University of Chicago nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.

import yaml
import math
import os.path
import chisubmit.core
from chisubmit.common.utils import set_datetime_timezone_utc
import urllib2
from chisubmit.core import ChisubmitException


class ChisubmitModelException(Exception):
    def __init__(self, message):
        Exception.__init__(self, message)


class Course(object):
    def __init__(self, course_id, name, extensions):
        self.id = course_id
        self.name = name
        self.extensions = extensions
        
        self.github_organization = None
        self.github_admins_team = None
        
        self.git_staging_username = None
        self.git_staging_hostname = None
        
        self.students = {}
        self.projects = {}
        self.teams = {}
        
        self.course_file = None
        
    def save(self, course_file = None):
        if self.course_file is None and course_file is None:
            raise ChisubmitModelException("Course object has no file associated with it.")

        if course_file is None:
            course_file = self.course_file
            
        try:
            f = open(course_file, 'w')
            yaml.dump(self, f)
            f.close()
        except IOError, ioe:
            raise ChisubmitException("Error when saving course to file %s: %s" % (course_file, ioe.meesage), ioe)
        
    @staticmethod
    def from_file(course_file):
        course = yaml.load(course_file)
        if type(course) != Course:
            raise ChisubmitModelException("Course file does not contain a Course object.")
        return course

    @staticmethod
    def from_url(course_url):
        try:
            req = urllib2.Request(course_url)
            response = urllib2.urlopen(req)
        except urllib2.HTTPError, he:
            raise ChisubmitException("Error when accessing %s: %s %s" % (course_url, he.code, he.msg), he)
        except urllib2.URLError, ue:
            raise ChisubmitException("Error when accessing %s: %s" % (course_url, ue.reason), ue)
        
        course = yaml.load(response)
        if type(course) != Course:
            raise ChisubmitModelException("Course file does not contain a Course object.")        
        
        return course
    
    @staticmethod
    def from_course_id(course_id):
        course_file = chisubmit.core.get_course_filename(course_id)
        if not os.path.exists(course_file):
            return None
        course_file = open(course_file)
        course_obj = Course.from_file(course_file)
        course_file.close()
        return course_obj
    
    def get_student(self, student_id):
        return self.students.get(student_id)
        
    def add_student(self, student):
        self.students[student.id] = student

    def get_project(self, project_id):
        return self.projects.get(project_id)

    def add_project(self, project):
        self.projects[project.id] = project

    def get_team(self, team_id):
        return self.teams.get(team_id)

    def add_team(self, team):
        self.teams[team.id] = team
        
        
class GradeComponent(object):
    def __init__(self, name, points):
        self.name = name
        self.points = points        


class Project(object):
    def __init__(self, project_id, name, deadline):
        self.id = project_id
        self.name = name
        self.deadline = deadline
        self.grade_components = []
        
    def add_grade_component(self, grade_component):
        self.grade_components.append(grade_component)

    # We need to do this because PyYAML doesn't load
    # timezone data when reading dates. 
    def get_deadline(self):
        return set_datetime_timezone_utc(self.deadline)
        
    def extensions_needed(self, submission_time):
        delta = (submission_time - self.get_deadline()).total_seconds()
        
        extensions_needed = math.ceil(delta / (3600.0 * 24))
        
        if extensions_needed <= 0:
            return 0
        else:
            return int(extensions_needed)        
        
    def get_grading_branch_name(self):
        return self.id + "-grading"
        

class Student(object):
    def __init__(self, student_id, first_name, last_name, email, github_id):
        self.id = student_id
        self.first_name = first_name
        self.last_name = last_name
        self.email = email
        self.github_id = github_id
        
        self.dropped = False
        
    def __repr__(self):
        return "<Student %s: %s>" % (self.id, self.get_full_name())
        
    def get_full_name(self, comma = False):
        if comma:
            return "%s, %s" % (self.last_name, self.first_name)
        else:
            return "%s %s" % (self.first_name, self.last_name)
            
        
   
class Grader(object):
    def __init__(self, student_id, first_name, last_name, email, github_id):
        self.id = student_id
        self.first_name = first_name
        self.last_name = last_name
        self.email = email
        self.github_id = github_id

        
class Team(object):
    def __init__(self, team_id):
        self.id = team_id

        self.students = []
        self.active = True
        self.github_repo = None
        self.github_team = None
        self.github_email_sent = False
        
        self.projects = {}

    def __repr__(self):
        return "<Team %s>" % self.id
        
    def add_student(self, student):
        self.students.append(student)
        
    def add_project(self, project):
        self.projects[project.id] = TeamProject(project)
        
        
class TeamProject(object):
    def __init__(self, project):
        self.project = project
        
        self.grader = None
        self.extensions_used = 0
        self.grades = {}
        
    def set_grade(self, grade_component, points):
        self.grades[grade_component] = points
    



#  Copyright (c) 2013-2014, The University of Chicago
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of The University of Chicago nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.

from chisubmit.common.utils import create_subparser, mkdatetime,\
    get_datetime_now_utc, convert_timezone_to_local
from chisubmit.core.model import Project, GradeComponent
from chisubmit.common import CHISUBMIT_FAIL, CHISUBMIT_SUCCESS
from chisubmit.core import ChisubmitException, handle_unexpected_exception


def create_project_subparsers(subparsers):
    subparser = create_subparser(subparsers, "project-create", cli_do__project_create)
    subparser.add_argument('project_id', type=str)
    subparser.add_argument('name', type=str)
    subparser.add_argument('deadline', type=mkdatetime)

    subparser = create_subparser(subparsers, "project-list", cli_do__project_list)
    subparser.add_argument('--ids', action="store_true")
    subparser.add_argument('--utc', action="store_true")
    
    subparser = create_subparser(subparsers, "project-grade-component-add", cli_do__project_grade_component_add)
    subparser.add_argument('project_id', type=str)
    subparser.add_argument('name', type=str)
    subparser.add_argument('points', type=int)

    subparser = create_subparser(subparsers, "project-deadline-show", cli_do__project_deadline_show)
    subparser.add_argument('project_id', type=str)
    subparser.add_argument('--utc', action="store_true")
    

def cli_do__project_create(course, args):
    project = Project(project_id = args.project_id,
                      name = args.name,
                      deadline = args.deadline)
    course.add_project(project)
    
    return CHISUBMIT_SUCCESS
    

def cli_do__project_list(course, args):
    project_ids = course.projects.keys()
    project_ids.sort()
    
    for project_id in project_ids:
        if args.ids:
            print project_id
        else:
            project = course.projects[project_id]
            
            if args.utc:
                deadline = project.get_deadline().isoformat(" ")
            else:
                deadline = convert_timezone_to_local(project.get_deadline()).isoformat(" ")
                
            fields = [project.id, deadline, project.name]
            
            print "\t".join(fields)

    return CHISUBMIT_SUCCESS

   
def cli_do__project_grade_component_add(course, args):
    project = course.get_project(args.project_id)
    if project is None:
        print "Project %s does not exist" % args.project_id
        return CHISUBMIT_FAIL

    grade_component = GradeComponent(args.name, args.points)
    project.add_grade_component(grade_component)    

    return CHISUBMIT_SUCCESS
    
    
def cli_do__project_deadline_show(course, args):
    project = course.get_project(args.project_id)
    if project is None:
        print "Project %s does not exist"
        return CHISUBMIT_FAIL
    
    now_utc = get_datetime_now_utc()
    now_local = convert_timezone_to_local(now_utc)
    
    deadline_utc = project.get_deadline()
    deadline_local = convert_timezone_to_local(deadline_utc)
        
    print project.name
    print
    if args.utc:
        print "      Now (Local): %s" % now_local.isoformat(" ")
        print " Deadline (Local): %s" % deadline_local.isoformat(" ")
        print
        print "        Now (UTC): %s" % now_utc.isoformat(" ")
        print "   Deadline (UTC): %s" % deadline_utc.isoformat(" ")
    else:
        print "      Now: %s" % now_local.isoformat(" ")
        print " Deadline: %s" % deadline_local.isoformat(" ")
        
    print 
    
    extensions = project.extensions_needed(now_utc)

    if extensions == 0:
        diff = deadline_utc - now_utc
    else:
        diff = now_utc - deadline_utc  
    
    days = diff.days
    hours = diff.seconds // 3600
    minutes = (diff.seconds//60)%60
    seconds = diff.seconds%60
    
    if extensions == 0:
        print "The deadline has not yet passed"
        print "You have %i days, %i hours, %i minutes, %i seconds left" % (days, hours, minutes, seconds)
    else:
        print "The deadline passed %i days, %i hours, %i minutes, %i seconds ago" % (days, hours, minutes, seconds)
        print "If you submit your project now, you will need to use %i extensions" % extensions
 
    return CHISUBMIT_SUCCESS
       
            



#  Copyright (c) 2013-2014, The University of Chicago
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of The University of Chicago nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.

from chisubmit.common.utils import create_subparser
from chisubmit.core.repos import GradingGitRepo
from chisubmit.common import CHISUBMIT_SUCCESS, CHISUBMIT_FAIL
from chisubmit.core import ChisubmitException, handle_unexpected_exception

def create_grader_subparsers(subparsers):
    subparser = create_subparser(subparsers, "grader-sync-grading-repo", cli_do__grader_sync_grading_repo)
    subparser.add_argument('team_id', type=str)

    subparser = create_subparser(subparsers, "grader-create-grading-branch", cli_do__grader_create_grading_branch)
    subparser.add_argument('team_id', type=str)
    subparser.add_argument('project_id', type=str)

    subparser = create_subparser(subparsers, "grader-push-grading-branch", cli_do__grader_push_grading_branch)
    subparser.add_argument('--staging', action="store_true")
    subparser.add_argument('--github', action="store_true")
    subparser.add_argument('team_id', type=str)
    subparser.add_argument('project_id', type=str)

    subparser = create_subparser(subparsers, "grader-pull-grading-branch", cli_do__grader_pull_grading_branch)
    subparser.add_argument('--staging', action="store_true")
    subparser.add_argument('--github', action="store_true")
    subparser.add_argument('team_id', type=str)
    subparser.add_argument('project_id', type=str)


def cli_do__grader_sync_grading_repo(course, args):
    team = course.get_team(args.team_id)
    if team is None:
        print "Team %s does not exist" % args.team_id
        return CHISUBMIT_FAIL
    
    try:
        repo = GradingGitRepo.get_grading_repo(course, team)
        
        if repo is None:
            repo = GradingGitRepo.create_grading_repo(course, team)
        else:
            repo.sync()
    except ChisubmitException, ce:
        raise ce # Propagate upwards, it will be handled by chisubmit_cmd
    except Exception, e:
        handle_unexpected_exception()

    return CHISUBMIT_SUCCESS

        
def cli_do__grader_create_grading_branch(course, args):
    team = course.get_team(args.team_id)
    if team is None:
        print "Team %s does not exist"
        return CHISUBMIT_FAIL
    
    project = course.get_project(args.project_id)
    if project is None:
        print "Project %s does not exist"
        return CHISUBMIT_FAIL
    
    try:
        repo = GradingGitRepo.get_grading_repo(course, team)
        
        if repo is None:
            print "%s does not have a grading repository" % team.id
            return CHISUBMIT_FAIL
        
        repo.create_grading_branch(project)
    except ChisubmitException, ce:
        raise ce # Propagate upwards, it will be handled by chisubmit_cmd
    except Exception, e:
        handle_unexpected_exception()

    return CHISUBMIT_SUCCESS


def cli_do__grader_push_grading_branch(course, args):
    team = course.get_team(args.team_id)
    if team is None:
        print "Team %s does not exist"
        return CHISUBMIT_FAIL
    
    project = course.get_project(args.project_id)
    if project is None:
        print "Project %s does not exist"
        return CHISUBMIT_FAIL
    
    try:    
        repo = GradingGitRepo.get_grading_repo(course, team)
        
        if repo is None:
            print "%s does not have a grading repository" % team.id
            return CHISUBMIT_FAIL
            
        if args.github:
            repo.push_grading_branch_to_github(project)
            
        if args.staging:
            repo.push_grading_branch_to_staging(project)
    except ChisubmitException, ce:
        raise ce # Propagate upwards, it will be handled by chisubmit_cmd
    except Exception, e:
        handle_unexpected_exception()

    return CHISUBMIT_SUCCESS

def cli_do__grader_pull_grading_branch(course, args):
    team = course.get_team(args.team_id)
    if team is None:
        print "Team %s does not exist"
        return CHISUBMIT_FAIL
    
    project = course.get_project(args.project_id)
    if project is None:
        print "Project %s does not exist"
        return CHISUBMIT_FAIL
    
    try:
        repo = GradingGitRepo.get_grading_repo(course, team)
        
        if repo is None:
            print "%s does not have a grading repository" % team.id
            return CHISUBMIT_FAIL
        
        if args.github:
            repo.pull_grading_branch_from_github(project)
        
        if args.staging:
            repo.pull_grading_branch_from_staging(project)
    except ChisubmitException, ce:
        raise ce # Propagate upwards, it will be handled by chisubmit_cmd
    except Exception, e:
        handle_unexpected_exception()
        
    return CHISUBMIT_SUCCESS
                



#  Copyright (c) 2013-2014, The University of Chicago
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of The University of Chicago nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.

import getpass

import chisubmit.core

from chisubmit.common.utils import create_subparser
from chisubmit.core.repos import GithubConnection
from chisubmit.common import CHISUBMIT_SUCCESS, CHISUBMIT_FAIL
from chisubmit.core import ChisubmitException, handle_unexpected_exception

def create_gh_subparsers(subparsers):
    subparser = create_subparser(subparsers, "gh-token-create", cli_do__gh_token_create)
    subparser.add_argument('--delete', action="store_true")
    
    
def cli_do__gh_token_create(course, args):
    
    try:
        username = raw_input("Enter your GitHub username: ")
        password = getpass.getpass("Enter your GitHub password: ")
    except KeyboardInterrupt, ki:
        exit(CHISUBMIT_FAIL)
    except Exception, e:
        handle_unexpected_exception()
    
    try:
        token = GithubConnection.get_token(username, password, delete = args.delete)
        
        if token is None:
            print "Unable to create token. Incorrect username/password."
        else:
            if args.delete:
                chisubmit.core.set_github_delete_token(token)
            else:
                chisubmit.core.set_github_token(token)
    
            print "The following token has been created: %s" % token
            print "chisubmit has been configured to use this token from now on."
    except ChisubmitException, ce:
        raise ce # Propagate upwards, it will be handled by chisubmit_cmd
    except Exception, e:
        handle_unexpected_exception()
            
    return CHISUBMIT_SUCCESS

        



#  Copyright (c) 2013-2014, The University of Chicago
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of The University of Chicago nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.

import chisubmit.core

from chisubmit.common.utils import create_subparser, set_datetime_timezone_utc, convert_timezone_to_local
from chisubmit.core.repos import GithubConnection
from chisubmit.common import CHISUBMIT_SUCCESS, CHISUBMIT_FAIL
from chisubmit.core import ChisubmitException

def create_submit_subparsers(subparsers):
    subparser = create_subparser(subparsers, "team-project-submit", cli_do__team_project_submit)
    subparser.add_argument('team_id', type=str)    
    subparser.add_argument('project_id', type=str)
    subparser.add_argument('commit', type=str)
    subparser.add_argument('extensions', type=int, default=0)
    subparser.add_argument('--force', action="store_true")
    subparser.add_argument('--yes', action="store_true")
    subparser.add_argument('--ignore-extensions', action="store_true", dest="ignore_extensions")
    
    
def cli_do__team_project_submit(course, args):
    project = course.get_project(args.project_id)
    if project is None:
        print "Project %s does not exist" % args.project_id
        return CHISUBMIT_FAIL
    
    team = course.get_team(args.team_id)
    if team is None:
        print "Team %s does not exist" % args.team_id
        return CHISUBMIT_FAIL
    
    extensions_requested = args.extensions
    
    github_access_token = chisubmit.core.get_github_token()
    
    if github_access_token is None:
        print "You have not created a GitHub access token."
        print "You can create one using 'chisubmit gh-token-create'"
        return CHISUBMIT_FAIL
    
    try:
        gh = GithubConnection(github_access_token, course.github_organization)
    except ChisubmitException, ce:
        print "Unable to connect to GitHub: %s" % ce.message
        return CHISUBMIT_FAIL

    commit = gh.get_commit(team, args.commit)
    
    if commit is None:
        print "Commit %s does not exist in repository" % commit
        return CHISUBMIT_FAIL
        
    commit_time_utc = set_datetime_timezone_utc(commit.commit.author.date)
    commit_time_local = convert_timezone_to_local(commit_time_utc)
    
    deadline_utc = project.get_deadline()
    deadline_local = convert_timezone_to_local(deadline_utc)
        
    extensions_needed = project.extensions_needed(commit_time_utc)
    
    extensions_bad = False
    if extensions_requested < extensions_needed:
        print
        print "The number of extensions you have requested is insufficient."
        print
        print "     Deadline (UTC): %s" % deadline_utc.isoformat()
        print "       Commit (UTC): %s" % commit_time_utc.isoformat()
        print 
        print "   Deadline (Local): %s" % deadline_local.isoformat()
        print "     Commit (Local): %s" % commit_time_local.isoformat()
        print 
        print "You need to request %s extensions." % extensions_needed
        extensions_bad = True
    elif extensions_requested > extensions_needed:
        print        
        print "The number of extensions you have requested is excessive."
        print
        print "     Deadline (UTC): %s" % deadline_utc.isoformat()
        print "       Commit (UTC): %s" % commit_time_utc.isoformat()
        print 
        print "   Deadline (Local): %s" % deadline_local.isoformat()
        print "     Commit (Local): %s" % commit_time_local.isoformat()
        print 
        print "You only need to request %s extensions." % extensions_needed
        extensions_bad = True

    if not args.ignore_extensions and extensions_bad:
        print
        print "You can use the --ignore-extensions option to submit regardless, but"
        print "you should get permission from the instructor before you do so."
        print
        return CHISUBMIT_FAIL
    elif args.ignore_extensions and extensions_bad:
        print
        print "WARNING: You are forcing a submission with an incorrect number"
        print "of extensions. Make sure you have approval from the instructor"
        print "to do this."
        
    tag_name = project.id
    submission_tag = gh.get_submission_tag(team, tag_name)
    
    if submission_tag is not None and not args.force:
        submission_commit = gh.get_commit(team, submission_tag.object.sha)
        print        
        print "Submission tag '%s' already exists" % tag_name
        print "It points to commit %s (%s)" % (submission_commit.commit.sha, submission_commit.commit.message)
        print "If you want to override this submission, please use the --force option"
        return CHISUBMIT_FAIL
    elif submission_tag is not None and args.force:
        submission_commit = gh.get_commit(team, submission_tag.object.sha)
        print
        print "WARNING: Submission tag '%s' already exists" % tag_name
        print "It currently points to commit %s...: %s" % (submission_commit.commit.sha[:8], submission_commit.commit.message)
        print "Make sure you want to overwrite the previous submission tag."
        
    print
    print "You are going to tag your code for %s as ready to grade." % project.name
    print "The commit you are submitting is the following:"
    print
    print "      Commit: %s" % commit.commit.sha
    print "        Date: %s" % commit.commit.author.date.isoformat()
    print "     Message: %s" % commit.commit.message
    print "      Author: %s <%s>" % (commit.commit.author.name, commit.commit.author.email)
    if not extensions_bad:
        print
        print "The number of extensions you are requesting (%i) is acceptable." % args.extensions
        print "Please note that this program does not check how many extensions"
        print "you have left. It only checks whether the number of extensions is"
        print "correct given the deadline for the project."
    
    print
    print "Are you sure you want to continue? (y/n): ", 
    
    if not args.yes:
        yesno = raw_input()
    else:
        yesno = 'y'
        print 'y'
    
    if yesno in ('y', 'Y', 'yes', 'Yes', 'YES'):
        message = "Extensions requested: %i\n" % args.extensions
        message += "Extensions needed: %i\n" % extensions_needed
        if extensions_bad:
            message += "Extensions bad: yes\n"
            
        if submission_tag is None:
            gh.create_submission_tag(team, tag_name, message, commit.commit.sha)
        else:
            gh.update_submission_tag(team, tag_name, message, commit.commit.sha)
            
        print
        print "Your submission has been completed."
        #print "You can use 'chisubmit team-project-submission-verify' to double-check"
        #print "that your code was correctly tagged as ready to grade."
        
    return CHISUBMIT_SUCCESS

        



#  Copyright (c) 2013-2014, The University of Chicago
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of The University of Chicago nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.

import os.path
from pkg_resources import Requirement, resource_filename
import ConfigParser
import shutil
import traceback
from chisubmit.common import CHISUBMIT_FAIL

DEFAULT_CHISUBMIT_DIR = os.path.expanduser("~/.chisubmit/")
DEFAULT_CONFIG_FILENAME = "chisubmit.conf"
DEFAULT_COURSE_FILENAME = "default_course"
GHTOKEN_FILENAME = "github_token"
GHDELETETOKEN_FILENAME = "github_delete_token"

chisubmit_dir = None
chisubmit_conf = None

class ChisubmitException(Exception):
    def __init__(self, message, original_exception = None):
        Exception.__init__(self, message)
        self.original_exception = original_exception
        if original_exception is not None:
            self.traceback = traceback.format_exc()
        else:
            self.traceback = None

    def print_exception(self):
        print self.traceback

def handle_unexpected_exception():
    print "ERROR: Unexpected exception"
    print traceback.format_exc()
    exit(CHISUBMIT_FAIL)

def init_chisubmit(base_dir = None, config_file = None):
    global chisubmit_dir, chisubmit_conf
    
    if base_dir is None:
        chisubmit_dir = DEFAULT_CHISUBMIT_DIR
    else:
        chisubmit_dir = os.path.expanduser(base_dir)
        
    if config_file is None:
        chisubmit_config_file = chisubmit_dir + DEFAULT_CONFIG_FILENAME
    else:
        chisubmit_config_file = os.path.expanduser(config_file)
    
    # Create chisubmit directory if it does not exist
    if not os.path.exists(chisubmit_dir):
        os.mkdir(chisubmit_dir)
        
    if not os.path.exists(chisubmit_dir + "/courses/"):        
        os.mkdir(chisubmit_dir + "/courses/")

    if not os.path.exists(chisubmit_dir + "/repositories/"):        
        os.mkdir(chisubmit_dir + "/repositories/")

    if not os.path.exists(chisubmit_config_file):
        example_conf = resource_filename(Requirement.parse("chisubmit"), "config/chisubmit.sample.conf")    
        shutil.copyfile(example_conf, chisubmit_config_file)   
    
    chisubmit_conf = ConfigParser.ConfigParser()
    chisubmit_conf.read(chisubmit_config_file)

def __get_file(filename):
    fname = chisubmit_dir + "/" + filename
    if not os.path.exists(fname):
        return None
    else:
        try:
            f = open(fname)
            contents = f.read().strip()
            return contents
        except IOError, ioe:
            raise ChisubmitException("Error when reading from file %s: %s" % (fname, ioe.meesage), ioe)        

def __set_file(filename, contents):
    fname = chisubmit_dir + "/" + filename
    try:
        f = open(fname, 'w')
        f.write(contents + "\n")
    except IOError, ioe:
        raise ChisubmitException("Error when writing to file %s: %s" % (fname, ioe.meesage), ioe)        
        
def get_default_course():
    return __get_file(DEFAULT_COURSE_FILENAME)

def set_default_course(course_id):
    __set_file(DEFAULT_COURSE_FILENAME, course_id)

def get_github_token():
    return __get_file(GHTOKEN_FILENAME)

def set_github_token(token):
    __set_file(GHTOKEN_FILENAME, token)

def get_github_delete_token():
    return __get_file(GHDELETETOKEN_FILENAME)

def set_github_delete_token(token):
    __set_file(GHDELETETOKEN_FILENAME, token)
    
def get_course_filename(course_id):
    return chisubmit_dir + "/courses/" + course_id + ".yaml"

    

    



#  Copyright (c) 2013-2014, The University of Chicago
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of The University of Chicago nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.

from chisubmit.common.utils import create_subparser
from chisubmit.core.model import Student
from chisubmit.common import CHISUBMIT_FAIL, CHISUBMIT_SUCCESS

def create_student_subparsers(subparsers):
    subparser = create_subparser(subparsers, "student-create", cli_do__student_create)
    subparser.add_argument('id', type=str)
    subparser.add_argument('first_name', type=str)
    subparser.add_argument('last_name', type=str)
    subparser.add_argument('email', type=str)
    subparser.add_argument('github_id', type=str)
    
    subparser = create_subparser(subparsers, "student-set-dropped", cli_do__student_set_dropped)
    subparser.add_argument('id', type=str)


def cli_do__student_create(course, args):
    student = Student(student_id = args.id,
                      first_name = args.first_name,
                      last_name = args.last_name,
                      email = args.email,
                      github_id = args.github_id)
    course.add_student(student)
    
    return CHISUBMIT_SUCCESS
    
    
def cli_do__student_set_dropped(course, args):
    student = course.get_student(args.id)
    if student is None:
        print "Student %s does not exist" % args.id
        return CHISUBMIT_FAIL    
    
    student.dropped = True
    
    return CHISUBMIT_SUCCESS
